var MGC = {};

//MGC Calculator
MGC.Calculator = function(id){
	"use strict";
	var thisCalc = this;
	/*
		-window values-
		The logic for these values is not currently working correctly:
			Have to keep the scale at 1
			Max and min values need to be mirrored. 
				(i.e) If xMax is 10, xMin needs to be -10			
	*/
	var xMax = 5;
	var xMaxVal = 40;
	var xScale = 1;
	var yMax = 5;
	var yMaxVal = 40;
	var yScale = 1;
	
	/*
		-start coordinates-
		The coordinate (0, 0) on the canvas is the 
		upper-left hand corner. This is updated
		according to the current scales and max/mins
	*/
	var xStart = 0;
	var yStart = 0;
	
	//arrays for drawing every function graph
	var funcArr = []; //array of functions
	var colorArr = []; //array of functions
	
	//canvas and context
	var canvas = null;
	var context = null;

	canvas = createCanvas(); //make a canvas for the graph output
	
	if(document.getElementById(id) === null){ //append canvas to the body
		document.body.appendChild(canvas);
	}
	else{ //append canvas to the element at id
		document.getElementById(id).appendChild(canvas);
	}
	
	context = getContext(canvas); //init context

	drawGraph(); //draw an empty grid
	
	/*start public methods*/	
	this.graphFunc = function(func, color){
		func = func.toLowerCase();
		
		if(cleanExp(func) !== null){
			funcArr.push(func); //save function
			colorArr.push(color); //save color
			drawGraph(); //draw new graph
			return true;
		}
		else{
			return false;
		}
		
	};
	
	this.clearFunc = function(func, color){
		var i = colorArr.indexOf(color);
		
		if(i > -1){
			funcArr.splice(i, 1); //remove function name
			colorArr.splice(i, 1); //remove color
			drawGraph(); //draw new graph
		}
	};
	
	this.setMax = function(value, axis){
			//value currently has to be symmetrical
			//also between 0 and 100
			if(value > 0 && value < 100){ 
				if(axis === "x"){
					xMax = value;
					xMaxVal = canvas.width / 2 / xMax;
				}				
				drawGraph();
			}
			
	};
	
	/*
		will eventually need public get and set 
		functions for all values:
		---scale
		---min
		---canvas size
	*/
	/*end public methods*/
	
	/*start private methods*/
	function createCanvas(){
		//create a canvas
		canvas = document.createElement("canvas");
		
		if(canvas === null){ //check canvas
			throw MGC.Errors.canvasErr;
		}
		
		//add and initalize canvas
		canvas.appendChild(document.createTextNode(MGC.Errors.tagErr));
		canvas.height = 400;
		canvas.width = 400;
		
		return canvas;
	}
	
	function getContext(){
		//get a context
		context = canvas.getContext("2d");
		
		if(context === null){ //check 2D context
			throw MGC.Error.contextErr;
		}
		
		return context;
	}
	
	function drawGraph(){
	
		//update starting positions
		xStart = 0.5 * canvas.width;
		yStart = 0.5 * canvas.height;
		
		//draw a white background
		context.fillStyle = "#ffffff";
		context.fillRect(0, 0, canvas.width, canvas.height);
		
		context.lineWidth = 1; //set line width for grid
		context.strokeStyle = "#cccccc";
		context.beginPath();
		
		//set up the grid
		for(var i = 0; i < canvas.width; i++){
			context.moveTo(i * (xMaxVal / xScale), 0);
			context.lineTo(i * (xMaxVal / xScale), canvas.width);
			
			context.moveTo(0, i * (yMaxVal / yScale));
			context.lineTo(canvas.height, i * (yMaxVal / yScale));
		}
		
		//draw the grid
		context.stroke();
		context.closePath();
		
		context.lineWidth = 5; //set line width for axises
		
		//draw x and y axis
		context.beginPath();
		context.moveTo(0, yStart);
		context.lineTo(canvas.width, yStart);
		context.moveTo(xStart, 0);
		context.lineTo(xStart, canvas.height);
		context.strokeStyle = "#000000";
		context.stroke();
		context.closePath();
		
		context.lineWidth = 1; //reset line width
		
		//draw the functions in the array
		for(i = 0; i < funcArr.length; i++){
			drawFunction(i);
		}
	}
	
	function drawFunction(i){
		var func = cleanExp(funcArr[i]);
		var color = colorArr[i];
		var x, y, max, min;
		
		if(func === null){ //invalid expression
			thisCalc.clearFunc(funcArr[i]);
			alert("BAD EXPRESSION");
			return;
		}
		
		max = (canvas.width - xStart) / xScale;
		min = -xStart / xScale;
		
		context.beginPath();
		context.lineWidth = 1;
		context.strokeStyle = color;
		
		//move to first coordinate
		x = xScale * min;
		
		try{ //expression might not evaluate
				y = xMaxVal * eval(func);
		}
		catch(err){
			thisCalc.clearFunc(funcArr[i]);
			alert("BAD EXPRESSION");
			return;
		}
			
		context.moveTo(xStart + x, yStart - y);
		
		for(var j = min + 1; j <= max; j++){
			x = xScale * j;
			
			try{ //expression might not evaluate
				y = xMaxVal * eval(func);
			}
			catch(err){
				thisCalc.clearFunc(funcArr[i]);
				alert("BAD EXPRESSION");
				return;
			}
			
			context.lineTo(xStart + x, yStart - y);
		
		}
		
		context.stroke();
		context.closePath();
	}
	
	function cleanExp(exp){
		var result = "";
		
		//remove spaces
		exp = exp.replace(/\s/g, '');
		
		//attempt to stop some code injection
		if(exp.indexOf("var") !== -1 || exp.indexOf("func") !== -1){
			//console.log(MGC.Errors.expWarn);
			return null;
		}
		else if(exp.indexOf("{") !== -1 || exp.indexOf("}") !== -1){
			//console.log(MGC.Errors.expWarn);
			return null;
		}
		else if(exp.indexOf("[") !== -1 || exp.indexOf("]") !== -1){
			//console.log(MGC.Errors.expWarn);
			return null;
		}
		else if(exp.indexOf("=") !== -1){
			//console.log(MGC.Errors.expWarn);
			return null;
		}
		else if(exp === ""){
			//console.log(MGC.Errors.expWarn);
			return null;
		}
		
		for(var i = 0; i < exp.length; i++){
			
			if(i+2 < exp.length && exp[i+1] === "^"){
				result += "pow(x, " + exp[i+2] + ")";
				i += 2;
			}
			else{
				result += exp[i];
			}
			
		}
		
		//scale x
		result = result.replace(/x/gi, "x/xMaxVal");
		
		//replace special numbers
		result = result.replace(/e/gi, "Math.E");
		result = result.replace(/pi/gi, "Math.PI");		
		
		return result;
	}
	
	function pow(x, e){
		return Math.pow(x, e);
	}
	
	function sin(x){
		return Math.sin(x);
	}
		
	function asin(x){
		return Math.asin(x);
	}
	
	function arcsin(x){
		return Math.asin(x);
	}
	
	function cos(x){
		return Math.cos(x);
	}
	
	function acos(x){
		return Math.acos(x);
	}
	
	function arccos(x){
		return Math.acos(x);
	}
	
	function tan(x){
		return Math.tan(x);
	}
	
	function atan(x){
		return Math.atan(x);
	}
	
	function arctan(x){
		return Math.atan(x);
	}
	
	function log(x){
		return Math.log(x);
	}
	/*end private methods*/
};

//MGC Errors
MGC.Errors = {
	tagErr		:	"Your browser does not seem to support the canvas tag.",
	canvasErr	:	"MGC_CANVAS_ERR: could not create a canvas element",
	contextErr	:	"MGC_CONTEXT_ERR: could not get a 2d context",
	expWarn		:	"MGC_EXPRESSION_WARNING: expression contains illegal characters"
};